<?php
namespace Affilicious\Detail\Model;

use Affilicious\Common\Helper\Assert_Helper;
use Affilicious\Common\Model\Simple_Value_Trait;

if (!defined('ABSPATH')) {
	exit('Not allowed to access pages directly.');
}

class Type
{
	const TEXT = 'text';
	const NUMBER = 'number';
	const BOOLEAN = 'boolean';
	const FILE = 'file';

	public static $all = [
		self::TEXT,
		self::NUMBER,
		self::BOOLEAN,
		self::FILE
	];

    use Simple_Value_Trait {
        Simple_Value_Trait::__construct as private set_value;
    }

	/**
     * Build a new text type.
     *
	 * @since 0.8
	 * @return Type
	 */
	public static function text()
	{
		return new self(self::TEXT);
	}

	/**
     * Build a new number type.
     *
	 * @since 0.8
	 * @return Type
	 */
	public static function number()
	{
		return new self(self::NUMBER);
	}

    /**
     * Build a new boolean type.
     *
     * @since 0.8.9
     * @return Type
     */
	public static function boolean()
    {
        return new self(self::BOOLEAN);
    }

	/**
     * Build a new file type.
     *
	 * @since 0.8
	 * @return Type
	 */
	public static function file()
	{
		return new self(self::FILE);
	}

	/**
	 * Build an array of all types.
	 *
	 * @since 0.9.2
	 * @return Type[]
	 */
	public static function all()
	{
		$all = [];

		foreach (self::$all as $value) {
			$all[] = new self($value);
		}

		return $all;
	}

    /**
     * @since 0.8
     * @param string $value
     */
	public function __construct($value)
	{
		Assert_Helper::is_string_not_empty($value, __METHOD__, 'The type must be a non empty string. Got: %s', '0.9.2');

		$this->set_value($value);
	}

    /**
     * Check if the type is a text.
     *
     * @since 0.8
     * @return bool
     */
    public function is_text()
    {
        return $this->value === self::TEXT;
    }

    /**
     * Check if the type is a number.
     *
     * @since 0.8
     * @return bool
     */
    public function is_number()
    {
        return $this->value === self::NUMBER;
    }

    /**
     * Check if the type is boolean.
     *
     * @since 0.8
     * @return bool
     */
    public function is_boolean()
    {
        return $this->value === self::BOOLEAN;
    }

    /**
     * Check if the type is a file.
     *
     * @since 0.8
     * @return bool
     */
    public function is_file()
    {
        return $this->value === self::FILE;
    }

    /**
     * Get the translated label for the type.
     *
     * @since 0.8
     * @return null|string The translated label if any.
     */
    public function get_label()
    {
        switch($this->value) {
            case self::TEXT:
                $label = __('Text', 'affilicious');
                break;
            case self::NUMBER:
                $label = __('Number', 'affilicious');
                break;
            case self::BOOLEAN:
                $label = __('Boolean', 'affilicious');
                break;
            case self::FILE:
                $label = __('File', 'affilicious');
                break;
            default:
                $label = null;
        }

	    $label = apply_filters('aff_detail_type_label', $label, $this->value);

        return $label;
    }
}
